@@ -16,6 +16,8 @@ email                : marco.hugentobler at sourcepole dot com
 #ifndef QGSABSTRACTGEOMETRYV2
 #define QGSABSTRACTGEOMETRYV2
 
+#include <functional>
+
 #include <QString>
 
 #include "qgis_core.h"
@@ -34,6 +36,8 @@ class QgsVertexIterator;
 class QPainter;
 class QDomDocument;
 class QDomElement;
+class QgsGeometryPartIterator;
+class QgsGeometryConstPartIterator;
 
 typedef QVector< QgsPoint > QgsPointSequence;
 #ifndef SIP_RUN
@@ -52,7 +56,6 @@ typedef QVector< QVector< QVector< QgsPoint > > > QgsCoordinateSequence;
  */
 class CORE_EXPORT QgsAbstractGeometry
 {
-    Q_GADGET
 
 #ifdef SIP_RUN
     SIP_CONVERT_TO_SUBCLASS_CODE
@@ -86,6 +89,9 @@ class CORE_EXPORT QgsAbstractGeometry
       sipType = 0;
     SIP_END
 #endif
+
+    Q_GADGET
+
   public:
 
     //! Segmentation tolerance as maximum angle or maximum difference between approximation and circle
@@ -104,6 +110,22 @@ class CORE_EXPORT QgsAbstractGeometry
     };
     Q_ENUM( SegmentationToleranceType )
 
+    //! Axis order for GML generation
+    enum AxisOrder
+    {
+
+      /**
+       * X comes before Y (or lon before lat)
+       */
+      XY = 0,
+
+      /**
+       * Y comes before X (or lat before lon)
+       */
+      YX
+    };
+    Q_ENUM( QgsAbstractGeometry::AxisOrder )
+
     /**
      * Constructor for QgsAbstractGeometry.
      */
@@ -163,13 +185,19 @@ class CORE_EXPORT QgsAbstractGeometry
      * Returns true if the geometry is 3D and contains a z-value.
      * \see isMeasure
      */
-    bool is3D() const;
+    bool is3D() const
+    {
+      return QgsWkbTypes::hasZ( mWkbType );
+    }
 
     /**
      * Returns true if the geometry contains m values.
      * \see is3D
      */
-    bool isMeasure() const;
+    bool isMeasure() const
+    {
+      return QgsWkbTypes::hasM( mWkbType );
+    }
 
     /**
      * Returns the closure of the combinatorial boundary of the geometry (ie the topological boundary of the geometry).
@@ -199,8 +227,8 @@ class CORE_EXPORT QgsAbstractGeometry
     /**
      * Returns a WKB representation of the geometry.
      * \see asWkt
-     * \see asGML2
-     * \see asGML3
+     * \see asGml2
+     * \see asGml3
      * \see asJson()
      * \since QGIS 3.0
      */
@@ -221,24 +249,26 @@ class CORE_EXPORT QgsAbstractGeometry
      * \param doc DOM document
      * \param precision number of decimal places for coordinates
      * \param ns XML namespace
+     * \param axisOrder Axis order for generated GML
      * \see asWkb()
      * \see asWkt()
      * \see asGml3()
      * \see asJson()
      */
-    virtual QDomElement asGml2( QDomDocument &doc, int precision = 17, const QString &ns = "gml" ) const = 0;
+    virtual QDomElement asGml2( QDomDocument &doc, int precision = 17, const QString &ns = "gml", AxisOrder axisOrder = QgsAbstractGeometry::AxisOrder::XY ) const = 0;
 
     /**
      * Returns a GML3 representation of the geometry.
      * \param doc DOM document
      * \param precision number of decimal places for coordinates
      * \param ns XML namespace
+     * \param axisOrder Axis order for generated GML
      * \see asWkb()
      * \see asWkt()
      * \see asGml2()
      * \see asJson()
      */
-    virtual QDomElement asGml3( QDomDocument &doc, int precision = 17, const QString &ns = "gml" ) const = 0;
+    virtual QDomElement asGml3( QDomDocument &doc, int precision = 17, const QString &ns = "gml", AxisOrder axisOrder = QgsAbstractGeometry::AxisOrder::XY ) const = 0;
 
     /**
      * Returns a GeoJSON representation of the geometry.
@@ -262,9 +292,7 @@ class CORE_EXPORT QgsAbstractGeometry
      * units (generally meters). If false, then z coordinates will not be changed by the
      * transform.
      */
-    virtual void transform( const QgsCoordinateTransform &ct,
-                            QgsCoordinateTransform::TransformDirection d = QgsCoordinateTransform::ForwardTransform,
-                            bool transformZ = false ) = 0;
+    virtual void transform( const QgsCoordinateTransform &ct, QgsCoordinateTransform::TransformDirection d = QgsCoordinateTransform::ForwardTransform, bool transformZ = false ) SIP_THROW( QgsCsException ) = 0;
 
     /**
      * Transforms the x and y components of the geometry using a QTransform object \a t.
@@ -338,7 +366,7 @@ class CORE_EXPORT QgsAbstractGeometry
      */
     virtual double closestSegment( const QgsPoint &pt, QgsPoint &segmentPt SIP_OUT,
                                    QgsVertexId &vertexAfter SIP_OUT,
-                                   int *leftOf SIP_OUT = nullptr, double epsilon = 4 * DBL_EPSILON ) const = 0;
+                                   int *leftOf SIP_OUT = nullptr, double epsilon = 4 * std::numeric_limits<double>::epsilon() ) const = 0;
 
     //low-level editing
 
@@ -468,7 +496,7 @@ class CORE_EXPORT QgsAbstractGeometry
      *
      * \since QGIS 3.0
      */
-    virtual bool removeDuplicateNodes( double epsilon = 4 * DBL_EPSILON, bool useZValues = false ) = 0;
+    virtual bool removeDuplicateNodes( double epsilon = 4 * std::numeric_limits<double>::epsilon(), bool useZValues = false ) = 0;
 
     /**
      * Returns approximate angle at a vertex. This is usually the average angle between adjacent
@@ -500,9 +528,9 @@ class CORE_EXPORT QgsAbstractGeometry
      * Adds a z-dimension to the geometry, initialized to a preset value.
      * \param zValue initial z-value for all nodes
      * \returns true on success
-     * \since QGIS 2.12
      * \see dropZValue()
      * \see addMValue()
+     * \since QGIS 2.12
      */
     virtual bool addZValue( double zValue = 0 ) = 0;
 
@@ -510,9 +538,9 @@ class CORE_EXPORT QgsAbstractGeometry
      * Adds a measure to the geometry, initialized to a preset value.
      * \param mValue initial m-value for all nodes
      * \returns true on success
-     * \since QGIS 2.12
      * \see dropMValue()
      * \see addZValue()
+     * \since QGIS 2.12
      */
     virtual bool addMValue( double mValue = 0 ) = 0;
 
@@ -534,6 +562,14 @@ class CORE_EXPORT QgsAbstractGeometry
      */
     virtual bool dropMValue() = 0;
 
+    /**
+     * Swaps the x and y coordinates from the geometry. This can be used
+     * to repair geometries which have accidentally had their latitude and longitude
+     * coordinates reversed.
+     * \since QGIS 3.2
+     */
+    virtual void swapXy() = 0;
+
     /**
      * Converts the geometry to a specified type.
      * \returns true if conversion was successful
@@ -543,6 +579,163 @@ class CORE_EXPORT QgsAbstractGeometry
 
 #ifndef SIP_RUN
 
+    /**
+     * Filters the vertices from the geometry in place, removing any which do not return true for the \a filter function
+     * check. Has no meaning when called on a single point geometry.
+     *
+     * Depending on the \a filter used, this may result in an invalid geometry.
+     *
+     * \note Not available in Python bindings
+     * \since QGIS 3.2
+     */
+    virtual void filterVertices( const std::function< bool( const QgsPoint & ) > &filter );
+
+    /**
+     * Transforms the vertices from the geometry in place, applying the \a transform function
+     * to every vertex.
+     *
+     * Depending on the \a transform used, this may result in an invalid geometry.
+     *
+     * Transform functions are not permitted to alter the dimensionality of vertices. If
+     * a transform which adds (or removes) z/m values is desired, first call the corresponding
+     * addZValue() or addMValue() function to change the geometry's dimensionality and then
+     * transform.
+     *
+     * \note Not available in Python bindings
+     * \since QGIS 3.4
+     */
+    virtual void transformVertices( const std::function< QgsPoint( const QgsPoint & ) > &transform );
+
+    /**
+     * \ingroup core
+     * The part_iterator class provides STL-style iterator for geometry parts.
+     * \since QGIS 3.4.3
+     */
+    class CORE_EXPORT part_iterator
+    {
+      private:
+
+        int mIndex = 0; //!< Current part in the geometry
+        QgsAbstractGeometry *mGeometry = nullptr;
+
+      public:
+        //! Create invalid iterator
+        part_iterator() = default;
+
+        //! Create part iterator for a geometry
+        part_iterator( QgsAbstractGeometry *g, int index );
+
+        /**
+         * The prefix ++ operator (++it) advances the iterator to the next part and returns an iterator to the new current part.
+         * Calling this function on iterator that is already past the last item leads to undefined results.
+         */
+        part_iterator &operator++();
+
+        //! The postfix ++ operator (it++) advances the iterator to the next part and returns an iterator to the previously current part.
+        part_iterator operator++( int );
+
+        //! Returns the current item.
+        QgsAbstractGeometry *operator*() const;
+
+        //! Returns the part number of the current item.
+        int partNumber() const;
+
+        bool operator==( part_iterator other ) const;
+        bool operator!=( part_iterator other ) const { return !( *this == other ); }
+    };
+
+    /**
+     * Returns STL-style iterator pointing to the first part of the geometry.
+     *
+     * \see parts_end()
+     * \see parts()
+     *
+     * \since QGIS 3.4.3
+     */
+    part_iterator parts_begin()
+    {
+      return part_iterator( this, 0 );
+    }
+
+    /**
+     * Returns STL-style iterator pointing to the imaginary part after the last part of the geometry.
+     *
+     * \see parts_begin()
+     * \see parts()
+     *
+     * \since QGIS 3.4.3
+     */
+    part_iterator parts_end();
+
+    /**
+     * Returns Java-style iterator for traversal of parts of the geometry. This iterator
+     * returns read-only references to parts and cannot be used to modify the parts.
+     *
+     * \note Not available in Python bindings
+     * \since QGIS 3.4.3
+     */
+    QgsGeometryConstPartIterator parts() const;
+
+    /**
+     * \ingroup core
+     * The part_iterator class provides STL-style iterator for const references to geometry parts.
+     * \since QGIS 3.4.3
+     */
+    class CORE_EXPORT const_part_iterator
+    {
+      private:
+
+        int mIndex = 0; //!< Current part in the geometry
+        const QgsAbstractGeometry *mGeometry = nullptr;
+
+      public:
+        //! Create invalid iterator
+        const_part_iterator() = default;
+
+        //! Create part iterator for a geometry
+        const_part_iterator( const QgsAbstractGeometry *g, int index );
+
+        /**
+         * The prefix ++ operator (++it) advances the iterator to the next part and returns an iterator to the new current part.
+         * Calling this function on iterator that is already past the last item leads to undefined results.
+         */
+        const_part_iterator &operator++();
+
+        //! The postfix ++ operator (it++) advances the iterator to the next part and returns an iterator to the previously current part.
+        const_part_iterator operator++( int );
+
+        //! Returns the current item.
+        const QgsAbstractGeometry *operator*() const;
+
+        //! Returns the part number of the current item.
+        int partNumber() const;
+
+        bool operator==( const_part_iterator other ) const;
+        bool operator!=( const_part_iterator other ) const { return !( *this == other ); }
+    };
+
+    /**
+     * Returns STL-style iterator pointing to the const first part of the geometry.
+     *
+     * \see const_parts_end()
+     *
+     * \since QGIS 3.4.3
+     */
+    const_part_iterator const_parts_begin() const
+    {
+      return const_part_iterator( this, 0 );
+    }
+
+    /**
+     * Returns STL-style iterator pointing to the imaginary const part after the last part of the geometry.
+     *
+     * \see const_parts_begin()
+     *
+     * \since QGIS 3.4.3
+     */
+    const_part_iterator const_parts_end() const;
+
+
     /**
      * \ingroup core
      * The vertex_iterator class provides STL-style iterator for vertices.
@@ -595,7 +788,11 @@ class CORE_EXPORT QgsAbstractGeometry
     };
 
     /**
-     * Returns STL-style iterator pointing to the first vertex of the geometry
+     * Returns STL-style iterator pointing to the first vertex of the geometry.
+     *
+     * \see vertices_end()
+     * \see vertices()
+     *
      * \since QGIS 3.0
      */
     vertex_iterator vertices_begin() const
@@ -604,7 +801,11 @@ class CORE_EXPORT QgsAbstractGeometry
     }
 
     /**
-     * Returns STL-style iterator pointing to the imaginary vertex after the last vertex of the geometry
+     * Returns STL-style iterator pointing to the imaginary vertex after the last vertex of the geometry.
+     *
+     * \see vertices_begin()
+     * \see vertices()
+     *
      * \since QGIS 3.0
      */
     vertex_iterator vertices_end() const
@@ -614,7 +815,60 @@ class CORE_EXPORT QgsAbstractGeometry
 #endif
 
     /**
-     * Returns Java-style iterator for traversal of vertices of the geometry
+     * Returns Java-style iterator for traversal of parts of the geometry. This iterator
+     * can safely be used to modify parts of the geometry.
+     *
+     * * Example:
+     * \code{.py}
+     *   # print the WKT representation of each part in a multi-point geometry
+     *   geometry = QgsMultiPoint.fromWkt( 'MultiPoint( 0 0, 1 1, 2 2)' )
+     *   for part in geometry.parts():
+     *       print(part.asWkt())
+     *
+     *   # single part geometries only have one part - this loop will iterate once only
+     *   geometry = QgsLineString.fromWkt( 'LineString( 0 0, 10 10 )' )
+     *   for part in geometry.parts():
+     *       print(part.asWkt())
+     *
+     *   # parts can be modified during the iteration
+     *   geometry = QgsMultiPoint.fromWkt( 'MultiPoint( 0 0, 1 1, 2 2)' )
+     *   for part in geometry.parts():
+     *       part.transform(ct)
+     *
+     *   # part iteration can also be combined with vertex iteration
+     *   geometry = QgsMultiPolygon.fromWkt( 'MultiPolygon((( 0 0, 0 10, 10 10, 10 0, 0 0 ),( 5 5, 5 6, 6 6, 6 5, 5 5)),((20 2, 22 2, 22 4, 20 4, 20 2)))' )
+     *   for part in geometry.parts():
+     *       for v in part.vertices():
+     *           print(v.x(), v.y())
+     *
+     * \endcode
+     *
+     * \see vertices()
+     * \since QGIS 3.4.3
+     */
+    QgsGeometryPartIterator parts();
+
+
+    /**
+     * Returns a read-only, Java-style iterator for traversal of vertices of all the geometry, including all geometry parts and rings.
+     *
+     * \warning The iterator returns a copy of individual vertices, and accordingly geometries cannot be
+     * modified using the iterator. See transformVertices() for a safe method to modify vertices "in-place".
+     *
+     * * Example:
+     * \code{.py}
+     *   # print the x and y coordinate for each vertex in a LineString
+     *   geometry = QgsLineString.fromWkt( 'LineString( 0 0, 1 1, 2 2)' )
+     *   for v in geometry.vertices():
+     *       print(v.x(), v.y())
+     *
+     *   # vertex iteration includes all parts and rings
+     *   geometry = QgsMultiPolygon.fromWkt( 'MultiPolygon((( 0 0, 0 10, 10 10, 10 0, 0 0 ),( 5 5, 5 6, 6 6, 6 5, 5 5)),((20 2, 22 2, 22 4, 20 4, 20 2)))' )
+     *   for v in geometry.vertices():
+     *       print(v.x(), v.y())
+     * \endcode
+     *
+     * \see parts()
      * \since QGIS 3.0
      */
     QgsVertexIterator vertices() const;
@@ -624,7 +878,6 @@ class CORE_EXPORT QgsAbstractGeometry
      * To create it, the geometry is default constructed and then the WKB is changed.
      * \see clone()
      * \since 3.0
-     * \note Not available in Python bindings
      */
     virtual QgsAbstractGeometry *createEmptyWithSameType() const = 0 SIP_FACTORY;
 
@@ -777,7 +1030,7 @@ class CORE_EXPORT QgsVertexIterator
       return g && g->vertices_end() != i;
     }
 
-    //! Return next vertex of the geometry (undefined behavior if hasNext() returns false before calling next())
+    //! Returns next vertex of the geometry (undefined behavior if hasNext() returns false before calling next())
     QgsPoint next();
 
 #ifdef SIP_RUN
@@ -801,4 +1054,103 @@ class CORE_EXPORT QgsVertexIterator
 
 };
 
+/**
+ * \ingroup core
+ * \brief Java-style iterator for traversal of parts of a geometry
+ * \since QGIS 3.4.3
+ */
+class CORE_EXPORT QgsGeometryPartIterator
+{
+  public:
+    //! Constructor for QgsGeometryPartIterator
+    QgsGeometryPartIterator() = default;
+
+    //! Constructs iterator for the given geometry
+    QgsGeometryPartIterator( QgsAbstractGeometry *geometry )
+      : g( geometry )
+      , i( g->parts_begin() )
+      , n( g->parts_end() )
+    {
+    }
+
+    //! Find out whether there are more parts
+    bool hasNext() const
+    {
+      return g && g->parts_end() != i;
+    }
+
+    //! Returns next part of the geometry (undefined behavior if hasNext() returns false before calling next())
+    QgsAbstractGeometry *next();
+
+#ifdef SIP_RUN
+    QgsGeometryPartIterator *__iter__();
+    % MethodCode
+    sipRes = sipCpp;
+    % End
+
+    SIP_PYOBJECT __next__();
+    % MethodCode
+    if ( sipCpp->hasNext() )
+      sipRes = sipConvertFromType( sipCpp->next(), sipType_QgsAbstractGeometry, NULL );
+    else
+      PyErr_SetString( PyExc_StopIteration, "" );
+    % End
+#endif
+
+  private:
+    QgsAbstractGeometry *g = nullptr;
+    QgsAbstractGeometry::part_iterator i, n;
+
+};
+
+
+/**
+ * \ingroup core
+ * \brief Java-style iterator for const traversal of parts of a geometry
+ * \since QGIS 3.4.3
+ */
+class CORE_EXPORT QgsGeometryConstPartIterator
+{
+  public:
+    //! Constructor for QgsGeometryConstPartIterator
+    QgsGeometryConstPartIterator() = default;
+
+    //! Constructs iterator for the given geometry
+    QgsGeometryConstPartIterator( const QgsAbstractGeometry *geometry )
+      : g( geometry )
+      , i( g->const_parts_begin() )
+      , n( g->const_parts_end() )
+    {
+    }
+
+    //! Find out whether there are more parts
+    bool hasNext() const
+    {
+      return g && g->const_parts_end() != i;
+    }
+
+    //! Returns next part of the geometry (undefined behavior if hasNext() returns false before calling next())
+    const QgsAbstractGeometry *next();
+
+#ifdef SIP_RUN
+    QgsGeometryConstPartIterator *__iter__();
+    % MethodCode
+    sipRes = sipCpp;
+    % End
+
+    SIP_PYOBJECT __next__();
+    % MethodCode
+    if ( sipCpp->hasNext() )
+      sipRes = sipConvertFromType( const_cast< QgsAbstractGeometry * >( sipCpp->next() ), sipType_QgsAbstractGeometry, NULL );
+    else
+      PyErr_SetString( PyExc_StopIteration, "" );
+    % End
+#endif
+
+  private:
+    const QgsAbstractGeometry *g = nullptr;
+    QgsAbstractGeometry::const_part_iterator i, n;
+
+};
+
 #endif //QGSABSTRACTGEOMETRYV2
