@@ -23,7 +23,7 @@ __copyright__ = '(C) 2015, Giovanni Manghi'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '26842169e9367434a27a70ab0df0bd4ba6ad1d7a'
+__revision__ = '2f64a3c4e74022c5555ee19862f034ffacc2b5fe'
 
 from qgis.core import (QgsProcessing,
                        QgsProcessingParameterDistance,
@@ -125,7 +125,7 @@ class Buffer(GdalAlgorithm):
             other_fields.append(f.name())
 
         if other_fields:
-            other_fields = ', {}'.format(','.join(other_fields))
+            other_fields = ',*'
         else:
             other_fields = ''
 
