@@ -23,11 +23,12 @@ __copyright__ = '(C) 2012, Victor Olaya'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '26842169e9367434a27a70ab0df0bd4ba6ad1d7a'
+__revision__ = '2f64a3c4e74022c5555ee19862f034ffacc2b5fe'
 
 import os
 
-from qgis.core import (QgsProcessingException,
+from qgis.core import (QgsProcessing,
+                       QgsProcessingException,
                        QgsProcessingParameterDefinition,
                        QgsProcessingParameterFeatureSource,
                        QgsProcessingParameterString,
@@ -37,7 +38,6 @@ from processing.algs.gdal.GdalUtils import GdalUtils
 
 
 class ogr2ogr(GdalAlgorithm):
-
     INPUT = 'INPUT'
     OPTIONS = 'OPTIONS'
     OUTPUT = 'OUTPUT'
@@ -47,7 +47,8 @@ class ogr2ogr(GdalAlgorithm):
 
     def initAlgorithm(self, config=None):
         self.addParameter(QgsProcessingParameterFeatureSource(self.INPUT,
-                                                              self.tr('Input layer')))
+                                                              self.tr('Input layer'),
+                                                              types=[QgsProcessing.TypeVector]))
 
         options_param = QgsProcessingParameterString(self.OPTIONS,
                                                      self.tr('Additional creation options'),
@@ -81,7 +82,7 @@ class ogr2ogr(GdalAlgorithm):
 
         output, outputFormat = GdalUtils.ogrConnectionStringAndFormat(outFile, context)
 
-        if outputFormat == 'SQLite' and os.path.isfile(output):
+        if outputFormat in ('SQLite', 'GPKG') and os.path.isfile(output):
             raise QgsProcessingException(self.tr('Output file "{}" already exists.'.format(output)))
 
         arguments = []
