@@ -23,7 +23,7 @@ __copyright__ = '(C) 2016, Médéric Ribreux'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = 'bc43194061452b37c8a22edcb7b380055a0c4236'
+__revision__ = '2f64a3c4e74022c5555ee19862f034ffacc2b5fe'
 
 
 def processInputs(alg, parameters, context, feedback):
@@ -43,4 +43,4 @@ def processOutputs(alg, parameters, context, feedback):
     if alg.parameterAsBool(parameters, '-l', context):
         dataType = 'line'
 
-    alg.exportVectorLayer(grassName, fileName, dataType)
+    alg.exportVectorLayer(grassName, fileName, dataType=dataType)
