@@ -23,7 +23,7 @@ __copyright__ = '(C) 2015, Arnaud Morvan'
 
 # This will get replaced with a git SHA1 when you do a git archive323
 
-__revision__ = '9b176802e5512bc310765d8ebe4e44b5f4663453'
+__revision__ = '2f64a3c4e74022c5555ee19862f034ffacc2b5fe'
 
 import os
 
@@ -157,9 +157,6 @@ class CheckValidity(QgisAlgorithm):
             if not geom.isNull() and not geom.isEmpty():
                 errors = list(geom.validateGeometry(method))
                 if errors:
-                    # QGIS method return a summary at the end
-                    if method == 1:
-                        errors.pop()
                     valid = False
                     reasons = []
                     for error in errors:
