@@ -23,12 +23,13 @@ __copyright__ = '(C) 2014, Piotr Pociask'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '8a899c8758b30adc3f8d6df682a5702dd3bfe6f0'
+__revision__ = '2f64a3c4e74022c5555ee19862f034ffacc2b5fe'
 
 from qgis.PyQt.QtCore import QCoreApplication
 from math import sqrt
 
-from qgis.core import (QgsFeature,
+from qgis.core import (QgsApplication,
+                       QgsFeature,
                        QgsFeatureSink,
                        QgsWkbTypes,
                        QgsProcessing,
@@ -75,10 +76,22 @@ class ConcaveHull(QgisAlgorithm):
         return 'concavehull'
 
     def displayName(self):
-        return self.tr('Concave hull')
+        return self.tr('Concave hull (alpha shapes)')
+
+    def shortDescription(self):
+        return self.tr('Creates a concave hull using the alpha shapes algorithm.')
+
+    def icon(self):
+        return QgsApplication.getThemeIcon("/algorithms/mAlgorithmConcaveHull.svg")
+
+    def svgIconPath(self):
+        return QgsApplication.iconPath("/algorithms/mAlgorithmConcaveHull.svg")
 
     def processAlgorithm(self, parameters, context, feedback):
         layer = self.parameterAsSource(parameters, ConcaveHull.INPUT, context)
+        if layer is None:
+            raise QgsProcessingException(self.invalidSourceError(parameters, self.INPUT))
+
         alpha = self.parameterAsDouble(parameters, self.ALPHA, context)
         holes = self.parameterAsBool(parameters, self.HOLES, context)
         no_multigeom = self.parameterAsBool(parameters, self.NO_MULTIGEOMETRY, context)
@@ -114,7 +127,7 @@ class ConcaveHull(QgisAlgorithm):
         counter = 50. / len(edges)
         i = 0
         ids = []
-        for id, max_len in list(edges.items()):
+        for id, max_len in edges.items():
             if feedback.isCanceled():
                 break
 
@@ -141,6 +154,8 @@ class ConcaveHull(QgisAlgorithm):
 
         (sink, dest_id) = self.parameterAsSink(parameters, self.OUTPUT, context,
                                                layer.fields(), QgsWkbTypes.Polygon, layer.sourceCrs())
+        if sink is None:
+            raise QgsProcessingException(self.invalidSinkError(parameters, self.OUTPUT))
 
         geom = feat.geometry()
         if no_multigeom and geom.isMultipart():
