@@ -23,7 +23,7 @@ __copyright__ = '(C) 2012, Victor Olaya'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '8a899c8758b30adc3f8d6df682a5702dd3bfe6f0'
+__revision__ = '2f64a3c4e74022c5555ee19862f034ffacc2b5fe'
 
 from qgis.PyQt.QtCore import QVariant
 from qgis.core import (QgsExpression,
@@ -32,6 +32,7 @@ from qgis.core import (QgsExpression,
                        QgsFeatureSink,
                        QgsField,
                        QgsDistanceArea,
+                       QgsProcessing,
                        QgsProcessingParameterFeatureSource,
                        QgsProcessingParameterEnum,
                        QgsProcessingParameterNumber,
@@ -71,7 +72,8 @@ class FieldsCalculator(QgisAlgorithm):
                            self.tr('Date')]
 
     def initAlgorithm(self, config=None):
-        self.addParameter(QgsProcessingParameterFeatureSource(self.INPUT, self.tr('Input layer')))
+        self.addParameter(QgsProcessingParameterFeatureSource(self.INPUT, self.tr('Input layer'),
+                                                              types=[QgsProcessing.TypeVector]))
         self.addParameter(QgsProcessingParameterString(self.FIELD_NAME,
                                                        self.tr('Result field name')))
         self.addParameter(QgsProcessingParameterEnum(self.FIELD_TYPE,
@@ -94,6 +96,9 @@ class FieldsCalculator(QgisAlgorithm):
 
     def processAlgorithm(self, parameters, context, feedback):
         source = self.parameterAsSource(parameters, self.INPUT, context)
+        if source is None:
+            raise QgsProcessingException(self.invalidSourceError(parameters, self.INPUT))
+
         layer = self.parameterAsVectorLayer(parameters, self.INPUT, context)
         field_name = self.parameterAsString(parameters, self.FIELD_NAME, context)
         field_type = self.TYPES[self.parameterAsEnum(parameters, self.FIELD_TYPE, context)]
@@ -118,6 +123,8 @@ class FieldsCalculator(QgisAlgorithm):
 
         (sink, dest_id) = self.parameterAsSink(parameters, self.OUTPUT, context,
                                                fields, source.wkbType(), source.sourceCrs())
+        if sink is None:
+            raise QgsProcessingException(self.invalidSinkError(parameters, self.OUTPUT))
 
         exp_context = self.createExpressionContext(parameters, context)
         if layer is not None:
