@@ -23,12 +23,13 @@ __copyright__ = '(C) 2012, Mathieu Pellerin'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '8a899c8758b30adc3f8d6df682a5702dd3bfe6f0'
+__revision__ = '2f64a3c4e74022c5555ee19862f034ffacc2b5fe'
 
 from qgis.core import (QgsDataSourceUri,
                        QgsFeatureSink,
                        QgsProcessingAlgorithm,
                        QgsVectorLayerExporter,
+                       QgsProcessing,
                        QgsProcessingException,
                        QgsProcessingParameterFeatureSource,
                        QgsProcessingParameterVectorLayer,
@@ -42,7 +43,6 @@ from processing.tools import spatialite
 
 
 class ImportIntoSpatialite(QgisAlgorithm):
-
     DATABASE = 'DATABASE'
     TABLENAME = 'TABLENAME'
     INPUT = 'INPUT'
@@ -65,17 +65,25 @@ class ImportIntoSpatialite(QgisAlgorithm):
         super().__init__()
 
     def initAlgorithm(self, config=None):
-        self.addParameter(QgsProcessingParameterFeatureSource(self.INPUT, self.tr('Layer to import')))
+        self.addParameter(QgsProcessingParameterFeatureSource(self.INPUT, self.tr('Layer to import'),
+                                                              types=[QgsProcessing.TypeVector]))
         self.addParameter(QgsProcessingParameterVectorLayer(self.DATABASE, self.tr('File database'), optional=False))
-        self.addParameter(QgsProcessingParameterString(self.TABLENAME, self.tr('Table to import to (leave blank to use layer name)'), optional=True))
-        self.addParameter(QgsProcessingParameterField(self.PRIMARY_KEY, self.tr('Primary key field'), None, self.INPUT, QgsProcessingParameterField.Any, False, True))
+        self.addParameter(
+            QgsProcessingParameterString(self.TABLENAME, self.tr('Table to import to (leave blank to use layer name)'),
+                                         optional=True))
+        self.addParameter(QgsProcessingParameterField(self.PRIMARY_KEY, self.tr('Primary key field'), None, self.INPUT,
+                                                      QgsProcessingParameterField.Any, False, True))
         self.addParameter(QgsProcessingParameterString(self.GEOMETRY_COLUMN, self.tr('Geometry column'), 'geom'))
         self.addParameter(QgsProcessingParameterString(self.ENCODING, self.tr('Encoding'), 'UTF-8', optional=True))
         self.addParameter(QgsProcessingParameterBoolean(self.OVERWRITE, self.tr('Overwrite'), True))
         self.addParameter(QgsProcessingParameterBoolean(self.CREATEINDEX, self.tr('Create spatial index'), True))
-        self.addParameter(QgsProcessingParameterBoolean(self.LOWERCASE_NAMES, self.tr('Convert field names to lowercase'), True))
-        self.addParameter(QgsProcessingParameterBoolean(self.DROP_STRING_LENGTH, self.tr('Drop length constraints on character fields'), False))
-        self.addParameter(QgsProcessingParameterBoolean(self.FORCE_SINGLEPART, self.tr('Create single-part geometries instead of multi-part'), False))
+        self.addParameter(
+            QgsProcessingParameterBoolean(self.LOWERCASE_NAMES, self.tr('Convert field names to lowercase'), True))
+        self.addParameter(QgsProcessingParameterBoolean(self.DROP_STRING_LENGTH,
+                                                        self.tr('Drop length constraints on character fields'), False))
+        self.addParameter(QgsProcessingParameterBoolean(self.FORCE_SINGLEPART,
+                                                        self.tr('Create single-part geometries instead of multi-part'),
+                                                        False))
 
     def flags(self):
         return super().flags() | QgsProcessingAlgorithm.FlagNoThreading
@@ -84,7 +92,13 @@ class ImportIntoSpatialite(QgisAlgorithm):
         return 'importintospatialite'
 
     def displayName(self):
-        return self.tr('Import into Spatialite')
+        return self.tr('Export to SpatiaLite')
+
+    def shortDescription(self):
+        return self.tr('Exports a vector layer to a SpatiaLite database')
+
+    def tags(self):
+        return self.tr('import,table,layer,into,copy').split(',')
 
     def processAlgorithm(self, parameters, context, feedback):
         database = self.parameterAsVectorLayer(parameters, self.DATABASE, context)
@@ -107,6 +121,8 @@ class ImportIntoSpatialite(QgisAlgorithm):
         encoding = self.parameterAsString(parameters, self.ENCODING, context)
 
         source = self.parameterAsSource(parameters, self.INPUT, context)
+        if source is None:
+            raise QgsProcessingException(self.invalidSourceError(parameters, self.INPUT))
 
         table = self.parameterAsString(parameters, self.TABLENAME, context)
         if table:
