@@ -23,7 +23,7 @@ __copyright__ = '(C) 2012, Victor Olaya'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = 'bc43194061452b37c8a22edcb7b380055a0c4236'
+__revision__ = '2f64a3c4e74022c5555ee19862f034ffacc2b5fe'
 
 import os
 import warnings
@@ -31,6 +31,7 @@ import warnings
 from qgis.PyQt import uic
 from qgis.PyQt.QtWidgets import QMenu, QAction, QInputDialog
 from qgis.PyQt.QtGui import QCursor
+from qgis.PyQt.QtCore import QCoreApplication
 
 from qgis.gui import QgsMessageBar
 from qgis.utils import iface
@@ -96,20 +97,27 @@ class ExtentSelectionPanel(BASE, WIDGET):
 
     def selectExtent(self):
         popupmenu = QMenu()
+        useCanvasExtentAction = QAction(
+            QCoreApplication.translate("ExtentSelectionPanel", 'Use Canvas Extent'),
+            self.btnSelect)
         useLayerExtentAction = QAction(
-            self.tr('Use layer/canvas extent'), self.btnSelect)
+            QCoreApplication.translate("ExtentSelectionPanel", 'Use Layer Extent…'),
+            self.btnSelect)
         selectOnCanvasAction = QAction(
-            self.tr('Select extent on canvas'), self.btnSelect)
+            self.tr('Select Extent on Canvas'), self.btnSelect)
 
-        popupmenu.addAction(useLayerExtentAction)
+        popupmenu.addAction(useCanvasExtentAction)
         popupmenu.addAction(selectOnCanvasAction)
+        popupmenu.addSeparator()
+        popupmenu.addAction(useLayerExtentAction)
 
         selectOnCanvasAction.triggered.connect(self.selectOnCanvas)
         useLayerExtentAction.triggered.connect(self.useLayerExtent)
+        useCanvasExtentAction.triggered.connect(self.useCanvasExtent)
 
         if self.param.flags() & QgsProcessingParameterDefinition.FlagOptional:
             useMincoveringExtentAction = QAction(
-                self.tr('Use min covering extent from input layers'),
+                self.tr('Use Min Covering Extent from Input Layers'),
                 self.btnSelect)
             useMincoveringExtentAction.triggered.connect(
                 self.useMinCoveringExtent)
@@ -121,11 +129,8 @@ class ExtentSelectionPanel(BASE, WIDGET):
         self.leText.setText('')
 
     def useLayerExtent(self):
-        CANVAS_KEY = 'Use canvas extent'
         extentsDict = {}
-        extentsDict[CANVAS_KEY] = {"extent": iface.mapCanvas().extent(),
-                                   "authid": iface.mapCanvas().mapSettings().destinationCrs().authid()}
-        extents = [CANVAS_KEY]
+        extents = []
         layers = QgsProcessingUtils.compatibleLayers(QgsProject.instance())
         for layer in layers:
             authid = layer.crs().authid()
@@ -137,10 +142,14 @@ class ExtentSelectionPanel(BASE, WIDGET):
             extents.append(layerName)
             extentsDict[layerName] = {"extent": layer.extent(), "authid": authid}
         (item, ok) = QInputDialog.getItem(self, self.tr('Select Extent'),
-                                          self.tr('Use extent from'), extents, False)
+                                          self.tr('Use extent from'), extents, 0, False)
         if ok:
             self.setValueFromRect(QgsReferencedRectangle(extentsDict[item]["extent"], QgsCoordinateReferenceSystem(extentsDict[item]["authid"])))
 
+    def useCanvasExtent(self):
+        self.setValueFromRect(QgsReferencedRectangle(iface.mapCanvas().extent(),
+                                                     iface.mapCanvas().mapSettings().destinationCrs()))
+
     def selectOnCanvas(self):
         canvas = iface.mapCanvas()
         canvas.setMapTool(self.tool)
