@@ -23,7 +23,7 @@ __copyright__ = '(C) 201, Victor Olaya'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = 'bc43194061452b37c8a22edcb7b380055a0c4236'
+__revision__ = '2f64a3c4e74022c5555ee19862f034ffacc2b5fe'
 
 import os
 import shutil
@@ -50,7 +50,7 @@ class AddScriptFromFileAction(ToolboxAction):
         files, _ = QFileDialog.getOpenFileNames(self.toolbox,
                                                 self.tr("Add script(s)"),
                                                 lastDir,
-                                                self.tr("Script files (*.py *.PY)"))
+                                                self.tr("Processing scripts (*.py *.PY)"))
         if files:
             settings.setValue("processing/lastScriptsDir", os.path.dirname(files[0]))
 
